#include <STC8H.H>
#include <intrins.h>
#define u8 unsigned char
#define u16 unsigned int

// ADS1115 地址定义（ADDR接GND时为0x48）
#define ADS_ADDR 0x48 << 1  // I2C地址需要左移1位

// LCD引脚定义
sbit LCD_E  = P3^2;
sbit LCD_DI = P3^3;
sbit LCD_CS1= P3^4;
sbit LCD_CS2= P3^5;
sbit LCD_RST= P3^6;
sbit LCD_RW = P3^7;

// 按键定义
sbit KEY_S1 = P7^0;

// 全局变量
bit key_flag = 0;    // 按键标志位
u8 test_step = 0;    // 测试步骤

// 延时函数（约1ms@11.0592MHz）
void delay_ms(u16 n) {
    while(n--) {
        u8 i = 250;
        while(--i);
        i = 247;
        while(--i);
    }
}

// I2C初始化
void I2C_Init() {
    P1M1 |= 0x03;    // P1.0,P1.1设置为开漏
    P1M0 |= 0x03;
    I2CCFG = 0xE0;   // 使能I2C，主机模式
    I2CMSAUX = 0x00;
    I2CMSCR = 0x73;  // 设置时钟
}

// I2C写操作
bit I2C_Write(u8 dat) {
    I2CTXD = dat;
    while(!(I2CMSST & 0x40)); // 等待传输完成
    return (I2CMSST & 0x80);  // 返回ACK状态
}

// ADS1115配置（通道选择）
u16 ADS1115_Read(u8 channel) {
    u16 result;
    
    // 配置寄存器（单次转换模式，±4.096V，1600SPS）
    I2CMSST = 0x01;  // 启动条件
    I2C_Write(ADS_ADDR);
    I2C_Write(0x01); // 指向配置寄存器
    
    switch(channel) {
        case 0: I2C_Write(0xC2); break; // AIN0 vs GND
        case 1: I2C_Write(0xD2); break; // AIN1 vs GND
        case 3: I2C_Write(0xF2); break; // AIN3 vs GND
    }
    I2C_Write(0x83); // 持续转换模式
    I2CMSST = 0x09;  // 停止条件
    
    delay_ms(2);      // 等待转换完成
    
    // 读取转换结果
    I2CMSST = 0x01;
    I2C_Write(ADS_ADDR | 0x01);
    result = I2CRXD << 8;
    result |= I2CRXD;
    I2CMSST = 0x09;
    
    return result;
}

// LCD写命令
void LCD_WriteCmd(u8 cmd) {
    while(LCD_RW);    // 等待空闲
    LCD_DI = 0;
    P2 = cmd;
    LCD_E = 1; _nop_(); _nop_();
    LCD_E = 0;
}

// LCD写数据
void LCD_WriteData(u8 dat) {
    while(LCD_RW);    // 等待空闲
    LCD_DI = 1;
    P2 = dat;
    LCD_E = 1; _nop_(); _nop_();
    LCD_E = 0;
}

// LCD初始化
void LCD_Init() {
    LCD_RST = 0;
    delay_ms(10);
    LCD_RST = 1;
    delay_ms(50);
    
    LCD_CS1 = 1;      // 选择左半屏
    LCD_CS2 = 1;      // 选择右半屏
    
    LCD_WriteCmd(0x3F); // 显示开
    LCD_WriteCmd(0xC0); // 起始行设置
}

// 继电器控制函数
void Relay_Control(u8 group1, u8 group2) {
    P0 = group1;      // 控制K1-K8（P0口）
    P4 = group2;      // 控制K9-K16（P4口）
    delay_ms(10);     // 确保继电器稳定
}

// 按键检测（带消抖）
void Key_Scan() {
    static u8 key_count = 0;
    
    if(!KEY_S1) {
        if(++key_count > 20 && !key_flag) {
            key_flag = 1;
            test_step = 1; // 开始测试流程
        }
    } else {
        key_count = 0;
        key_flag = 0;
    }
}

// 测试流程控制
void Test_Process() {
    static u8 phase = 0;
    u16 adc_value;
    float voltage;
    
    switch(test_step) {
        case 1: // Vos测试
            if(phase == 0) {
                Relay_Control(0x00, 0xA1); // K9(0x01), K10(0x02), K14(0x20) → 0x23?
                adc_value = ADS1115_Read(0);
                phase = 1;
            } else {
                Relay_Control(0x00, 0x00);
                // 计算并显示Vos...
                test_step = 2;
                phase = 0;
            }
            break;
            
        case 2: // IB测试
            if(phase == 0) {
                Relay_Control(0x00, 0x21); // K9, K14
                adc_value = ADS1115_Read(1);
                phase = 1;
            } else if(phase == 1) {
                Relay_Control(0x00, 0x12); // K10, K14
                adc_value = ADS1115_Read(1);
                phase = 0;
                test_step = 3;
            }
            break;
            
        // 其他测试步骤类似处理...
    }
}

void main() {
    // 初始化IO口模式
    P0M0 = 0xFF; P0M1 = 0x00; // P0推挽输出
    P4M0 = 0xFF; P4M1 = 0x00; // P4推挽输出
    P3M0 = 0xEC; P3M1 = 0x00; // P3.2-7推挽输出
    
    I2C_Init();
    LCD_Init();
    
    while(1) {
        Key_Scan();
        if(test_step) {
            Test_Process();
            delay_ms(1000);
        }
    }
}